@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 from plone.app.contenttypes import _
 from plone.app.textfield import RichText as RichTextField
+from plone.app.z3cform.widget import RichTextFieldWidget
+from plone.autoform import directives as form
 from plone.autoform.interfaces import IFormFieldProvider
 from plone.autoform.view import WidgetsView
 from plone.dexterity.interfaces import IDexterityContent
@@ -15,9 +17,10 @@ class IRichText(model.Schema):
 
     text = RichTextField(
         title=_(u'Text'),
-        description=u"",
+        description=u'',
         required=False,
     )
+    form.widget('text', RichTextFieldWidget)
     model.primary('text')
 
 
