@@ -5,9 +5,6 @@ from plone.app.contenttypes.utils import human_readable_size
 
 class FileView(Utils):
 
-    def __init__(self, context, request):
-        super(FileView, self).__init__(context, request)
-
     def is_videotype(self):
         ct = self.context.file.contentType
         return 'video/' in ct
