@@ -1,14 +1,4 @@
 # -*- coding: utf-8 -*-
-from Products.Archetypes.ExtensibleMetadata import ExtensibleMetadata
-from Products.CMFCore.interfaces import IPropertiesTool
-from Products.CMFCore.utils import getToolByName
-from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
-from Products.CMFPlone import PloneMessageFactory as _
-from Products.Five.browser import BrowserView
-from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
-from Products.PluginIndexes.UUIDIndex.UUIDIndex import UUIDIndex
-from Products.contentmigration.utils import patch, undoPatch
-from Products.statusmessages.interfaces import IStatusMessage
 from datetime import datetime
 from datetime import timedelta
 from plone.app.contenttypes.content import Document
@@ -19,24 +9,26 @@ from plone.app.contenttypes.content import Link
 from plone.app.contenttypes.content import NewsItem
 from plone.app.contenttypes.migration import dxmigration
 from plone.app.contenttypes.migration import migration
-from plone.app.contenttypes.migration.patches import \
-    patched_insertForwardIndexEntry
-from plone.app.contenttypes.migration.utils import HAS_LINGUA_PLONE
+from plone.app.contenttypes.migration.patches import patch_before_migration
+from plone.app.contenttypes.migration.patches import undo_patch_after_migration
 from plone.app.contenttypes.migration.utils import installTypeIfNeeded
 from plone.app.contenttypes.migration.utils import isSchemaExtended
 from plone.app.contenttypes.migration.utils import restore_references
 from plone.app.contenttypes.migration.utils import store_references
 from plone.app.contenttypes.migration.vocabularies import ATCT_LIST
-from plone.app.contenttypes.utils import DEFAULT_TYPES
 from plone.app.contenttypes.upgrades import use_new_view_names
-from plone.app.controlpanel.editing import IEditingSchema
+from plone.app.contenttypes.utils import DEFAULT_TYPES
 from plone.browserlayer.interfaces import ILocalBrowserLayerType
-from plone.dexterity.content import DexterityContent
 from plone.dexterity.interfaces import IDexterityContent
 from plone.dexterity.interfaces import IDexterityFTI
-from plone.registry.interfaces import IRegistry
 from plone.z3cform.layout import wrap_form
 from pprint import pformat
+from Products.CMFCore.utils import getToolByName
+from Products.CMFPlone import PloneMessageFactory as _
+from Products.CMFPlone.utils import get_installer
+from Products.Five.browser import BrowserView
+from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
+from Products.statusmessages.interfaces import IStatusMessage
 from z3c.form import button
 from z3c.form import field
 from z3c.form import form
@@ -45,13 +37,13 @@ from z3c.form.interfaces import HIDDEN_MODE
 from zExceptions import NotFound
 from zope import schema
 from zope.component import getMultiAdapter
-from zope.component import getUtility
 from zope.component import queryUtility
 from zope.interface import Interface
 
 import logging
 import pkg_resources
 
+
 try:
     pkg_resources.get_distribution('collective.contentleadimage')
 except pkg_resources.DistributionNotFound:
@@ -61,22 +53,12 @@ else:
 
 logger = logging.getLogger(__name__)
 
-PATCH_NOTIFY = [
-    DexterityContent,
-    DefaultDublinCoreImpl,
-    ExtensibleMetadata
-]
 
 # Average time to migrate one archetype object, in milliseconds.
 # This very much depends on the size of the object and system-speed
 ONE_OBJECT_MIGRATION_TIME = 500
 
 
-def pass_fn(*args, **kwargs):
-    """Empty function used for patching."""
-    pass
-
-
 class FixBaseClasses(BrowserView):
 
     def __call__(self):
@@ -84,7 +66,7 @@ class FixBaseClasses(BrowserView):
         Instances before version 1.0b1 had no base-class.
         To update them call @@fix_base_classes on your site-root.
         """
-        out = ""
+        out = ''
         portal_types = [
             ('Document', Document),
             ('File', File),
@@ -93,10 +75,8 @@ class FixBaseClasses(BrowserView):
             ('Link', Link),
             ('News Item', NewsItem),
         ]
-        catalog = getToolByName(self.context, "portal_catalog")
+        catalog = getToolByName(self.context, 'portal_catalog')
         query = {}
-        if HAS_LINGUA_PLONE and 'Language' in catalog.indexes():
-            query['Language'] = 'all'
         for portal_type, portal_type_class in portal_types:
             query['portal_type'] = portal_type
             results = catalog(query)
@@ -104,13 +84,14 @@ class FixBaseClasses(BrowserView):
                 try:
                     obj = brain.getObject()
                 except (KeyError, NotFound):
+                    logger.exception('Can not resolve object from brain.')
                     continue
                 if IDexterityContent.providedBy(obj):
                     object_class_name = obj.__class__.__name__
                     target_class_name = portal_type_class.__name__
                     if not object_class_name == target_class_name:
                         obj.__class__ = portal_type_class
-                        out += "Make %s use %s\n as base class." % (
+                        out += 'Make {0} use {1}\n as base class.'.format(
                             obj.Title(),
                             portal_type_class.__name__,
                         )
@@ -126,14 +107,15 @@ class MigrateFromATContentTypes(BrowserView):
 
     def __call__(self,
                  migrate=False,
-                 content_types="all",
+                 content_types='all',
                  migrate_schemaextended_content=False,
                  migrate_references=True,
-                 from_form=False):
+                 from_form=False,
+                 reindex_catalog=True,
+                 patch_searchabletext=False,
+                 ):
 
         portal = self.context
-        if content_types == 'all':
-            content_types = DEFAULT_TYPES
 
         if not from_form and migrate not in ['1', 'True', 'true', 1]:
             url1 = '{0}/@@migrate_from_atct?migrate=1'.format(
@@ -148,7 +130,7 @@ class MigrateFromATContentTypes(BrowserView):
             return msg
 
         helpers = getMultiAdapter((portal, self.request),
-                                  name="atct_migrator_helpers")
+                                  name='atct_migrator_helpers')
         if helpers.linguaplone_installed():
             msg = 'Warning\n'
             msg += 'Migration aborted since Products.LinguaPlone is '
@@ -160,44 +142,33 @@ class MigrateFromATContentTypes(BrowserView):
         stats_before = self.stats()
         starttime = datetime.now()
 
+        self.request['plone.app.contenttypes_migration_running'] = True
+
+        msg = 'Starting Migration\n\n'
+        msg += '\n-----------------------------\n'
+        msg += 'Content statictics:\n'
+        msg += pformat(stats_before)
+        msg += '\n-----------------------------\n'
+        msg += 'Types to be migrated:\n'
+        msg += pformat(content_types)
+        msg += '\n-----------------------------\n'
+        logger.info(msg)
+
         # store references on the portal
         if migrate_references:
             store_references(portal)
         catalog = portal.portal_catalog
 
-        # switch linkintegrity temp off
-        ptool = queryUtility(IPropertiesTool)
-        site_props = getattr(ptool, 'site_properties', None)
-        link_integrity_in_props = False
-        if site_props and site_props.hasProperty(
-                'enable_link_integrity_checks'):
-            link_integrity_in_props = True
-            link_integrity = site_props.getProperty(
-                'enable_link_integrity_checks', False)
-            site_props.manage_changeProperties(
-                enable_link_integrity_checks=False)
-        else:
-            # Plone 5
-            registry = getUtility(IRegistry)
-            editing_settings = registry.forInterface(
-                IEditingSchema, prefix='plone')
-            link_integrity = editing_settings.enable_link_integrity_checks
-            editing_settings.enable_link_integrity_checks = False
-
-        # switch of setModificationDate on changes
-        self.patchNotifyModified()
-
-        # patch UUIDIndex
-        patch(
-            UUIDIndex,
-            'insertForwardIndexEntry',
-            patched_insertForwardIndexEntry)
+        # Patch various things that make migration harder
+        (link_integrity,
+         queue_indexing,
+         patch_searchabletext) = patch_before_migration(patch_searchabletext)
 
         not_migrated = []
         migrated_types = {}
 
         for (k, v) in ATCT_LIST.items():
-            if k not in content_types:
+            if content_types != 'all' and k not in content_types:
                 not_migrated.append(k)
                 continue
             # test if the ct is extended beyond blobimage and blobfile
@@ -209,14 +180,15 @@ class MigrateFromATContentTypes(BrowserView):
                 'object_provides': v['iface'].__identifier__,
                 'meta_type': v['old_meta_type'],
             }
-            if HAS_LINGUA_PLONE and 'Language' in catalog.indexes():
-                query['Language'] = 'all'
             amount_to_be_migrated = len(catalog(query))
             starttime_for_current = datetime.now()
-            logger.info("Start migrating %s objects from %s to %s" % (
-                amount_to_be_migrated,
-                v['old_meta_type'],
-                v['type_name']))
+            logger.info(
+                'Start migrating {0} objects from {1} to {2}'.format(
+                    amount_to_be_migrated,
+                    v['old_meta_type'],
+                    v['type_name'],
+                )
+            )
             installTypeIfNeeded(v['type_name'])
 
             # call the migrator
@@ -225,11 +197,13 @@ class MigrateFromATContentTypes(BrowserView):
             # logging
             duration_current = datetime.now() - starttime_for_current
             duration_human = str(timedelta(seconds=duration_current.seconds))
-            logger.info("Finished migrating %s objects from %s to %s in %s" % (
-                amount_to_be_migrated,
-                v['old_meta_type'],
-                v['type_name'],
-                duration_human))
+            logger.info(
+                'Finished migrating {0} objects from {1} to {2} in {3}'.format(
+                    amount_to_be_migrated,
+                    v['old_meta_type'],
+                    v['type_name'],
+                    duration_human),
+            )
 
             # some data for the results-page
             migrated_types[k] = {}
@@ -243,34 +217,28 @@ class MigrateFromATContentTypes(BrowserView):
         # make sure the view-methods on the plone site are updated
         use_new_view_names(portal, types_to_fix=['Plone Site'])
 
-        catalog.clearFindAndRebuild()
+        if reindex_catalog:
+            catalog.clearFindAndRebuild()
 
         # restore references
         if migrate_references:
             restore_references(portal)
 
-        # switch linkintegrity back to what it was before migrating
-        if link_integrity_in_props:
-            site_props.manage_changeProperties(
-                enable_link_integrity_checks=link_integrity
-            )
-        else:
-            editing_settings.enable_link_integrity_checks = link_integrity
-
-        # switch on setModificationDate on changes
-        self.resetNotifyModified()
-
-        # unpatch UUIDIndex
-        undoPatch(UUIDIndex, 'insertForwardIndexEntry')
+        # Revert to the original state
+        undo_patch_after_migration(
+            link_integrity, queue_indexing, patch_searchabletext)
 
         duration = str(timedelta(seconds=(datetime.now() - starttime).seconds))
         if not_migrated:
-            msg = ("The following types were not migrated: \n %s"
-                   % "\n".join(not_migrated))
+            msg = (
+                'The following types were not migrated: \n {0}'.format(
+                    '\n'.join(not_migrated)
+                )
+            )
         else:
-            msg = "Migration successful\n\n"
+            msg = 'Migration successful\n\n'
         msg += '\n-----------------------------\n'
-        msg += 'Migration finished in: %s' % duration
+        msg += 'Migration finished in: {0}'.format(duration)
         msg += '\n-----------------------------\n'
         msg += 'Migration statictics:\n'
         msg += pformat(migrated_types)
@@ -297,11 +265,8 @@ class MigrateFromATContentTypes(BrowserView):
 
     def stats(self):
         results = {}
-        query = {}
         catalog = self.context.portal_catalog
-        if HAS_LINGUA_PLONE and 'Language' in catalog.indexes():
-            query['Language'] = 'all'
-        for brain in catalog(query):
+        for brain in catalog():
             try:
                 classname = brain.getObject().__class__.__name__
             except (KeyError, NotFound):
@@ -309,58 +274,61 @@ class MigrateFromATContentTypes(BrowserView):
             results[classname] = results.get(classname, 0) + 1
         return results
 
-    def patchNotifyModified(self):
-        """Patch notifyModified to prevent setModificationDate() on changes
-
-        notifyModified lives in several places and is also used on folders
-        when their content changes.
-        So when we migrate Documents before Folders the folders
-        ModifiedDate gets changed.
-        """
-        for klass in PATCH_NOTIFY:
-            patch(klass, 'notifyModified', pass_fn)
-
-    def resetNotifyModified(self):
-        """reset notifyModified to old state"""
-        for klass in PATCH_NOTIFY:
-            undoPatch(klass, 'notifyModified')
-
 
 class IATCTMigratorForm(Interface):
 
     content_types = schema.List(
-        title=u"Existing content that can be migrated",
-        description=u"Select which content types you want to migrate",
+        title=u'Existing content that can be migrated',
+        description=u'Select which content types you want to migrate',
         value_type=schema.Choice(
-            vocabulary="plone.app.contenttypes.migration.atctypes",
+            vocabulary='plone.app.contenttypes.migration.atctypes',
         ),
         required=False,
     )
 
     migrate_references = schema.Bool(
-        title=u"Migrate references?",
+        title=u'Migrate references?',
         description=(
-            u"Select this option to migrate references."
+            u'Select this option to migrate references.'
         ),
-        default=True
+        default=True,
+        required=False,
     )
 
     extended_content = schema.List(
         title=(
-            u"Migrate content that was extended "
-            u"using archetypes.schemaextender?"
+            u'Migrate content that was extended '
+            u'using archetypes.schemaextender?'
         ),
         description=(
-            u"Warning: You will loose all data in the extended fields!"
+            u'Warning: You will loose all data in the extended fields!'
         ),
         value_type=schema.Choice(
-            vocabulary="plone.app.contenttypes.migration.extendedtypes",
+            vocabulary='plone.app.contenttypes.migration.extendedtypes',
         ),
         required=False,
     )
 
+    reindex_catalog = schema.Bool(
+        title=_(u'Rebuild the catalog after the migration.'),
+        description=_(u'This operation can take a very long time.'),
+        default=True,
+        required=False,
+    )
+
+    patch_searchabletext = schema.Bool(
+        title=_(u'Disable reindexing objects during migration?'),
+        description=_(
+            u'This can speed up your migration a lot if you have a lot of files with searchable text.'  # noqa: E501
+        ),
+        default=False,
+        required=False,
+    )
+
 
 class ATCTMigratorForm(form.Form):
+    template = ViewPageTemplateFile('atct_migrator.pt')
+    results_template = ViewPageTemplateFile('atct_migrator_results.pt')
 
     fields = field.Fields(IATCTMigratorForm)
     fields['content_types'].widgetFactory = CheckBoxFieldWidget
@@ -368,6 +336,8 @@ class ATCTMigratorForm(form.Form):
     ignoreContext = True
     enableCSRFProtection = True
 
+    results = None
+
     @button.buttonAndHandler(u'Migrate', name='migrate')
     def handle_migrate(self, action):
         data, errors = self.extractData()
@@ -383,26 +353,34 @@ class ATCTMigratorForm(form.Form):
             (context, self.request),
             name=u'migrate_from_atct'
         )
-        # call the migration-view above to actually migrate stuff.
-        results = migration_view(
+        # store results where `render` can find them
+        self.results = migration_view(
             content_types=content_types,
             migrate_schemaextended_content=True,
             migrate_references=data['migrate_references'],
             from_form=True,
+            reindex_catalog=data['reindex_catalog'],
+            patch_searchabletext=data['patch_searchabletext'],
         )
-        sdm = getToolByName(context, "session_data_manager")
-        session = sdm.getSessionData(create=True)
-        session.set("atct_migrator_results", results)
-        url = context.absolute_url()
-        self.request.response.redirect(url + "/@@atct_migrator_results")
 
     def updateActions(self):
         super(ATCTMigratorForm, self).updateActions()
-        self.actions['migrate'].addClass("btn-danger")
+        self.actions['migrate'].addClass('btn-danger')
 
     def updateWidgets(self):
-        """ Overload this to hide empty widget """
+        """ Overload to set defaults and hide empty widgets. """
         form.Form.updateWidgets(self)
+        # 1. Select all items in the checkboxwidget
+        ct_widget = self.widgets['content_types']
+        # Only override when rendering the form. When you unselect
+        # everything the value will be [], not ())
+        if ct_widget.value == ():
+            ct_widget.value = list(ATCT_LIST)
+            # Call update again since the list-items checked-states are
+            # assigned in z3c.form.browser.checkbox.CheckBoxWidget.update
+            # using the widget.value.
+            ct_widget.update()
+        # 2. Hide empty widget
         for title, widget in self.widgets.items():
             if title not in ('content_types', 'extended_content'):
                 continue
@@ -410,11 +388,11 @@ class ATCTMigratorForm(form.Form):
                 # the vocabulary is empty, we hide the widget
                 widget.mode = HIDDEN_MODE
 
-
-ATCTMigrator = wrap_form(
-    ATCTMigratorForm,
-    index=ViewPageTemplateFile('atct_migrator.pt')
-)
+    def render(self):
+        if self.results:
+            return self.results_template()
+        else:
+            return super(ATCTMigratorForm, self).render()
 
 
 class IBaseClassMigratorForm(Interface):
@@ -428,11 +406,11 @@ class IBaseClassMigratorForm(Interface):
         required=True,
     )
     migrate_to_folderish = schema.Bool(
-        title=u"Migrate to folderish type?",
+        title=u'Migrate to folderish type?',
         description=(
-            u"Select this option if you changed a type from being "
-            u"itemish to being folderish but the class of the type is still "
-            u"the same."
+            u'Select this option if you changed a type from being '
+            u'itemish to being folderish but the class of the type is still '
+            u'the same.'
         ),
         default=False,
     )
@@ -457,7 +435,7 @@ class BaseClassMigratorForm(form.Form):
             return
 
         migrate_to_folderish = data.get('changed_base_classes', False)
-        catalog = getToolByName(self.context, "portal_catalog")
+        catalog = getToolByName(self.context, 'portal_catalog')
         migrated = []
         not_migrated = []
         for brain in catalog():
@@ -494,10 +472,8 @@ class ATCTMigratorHelpers(BrowserView):
 
     def objects_to_be_migrated(self):
         """ Return the number of AT objects in the portal """
-        catalog = getToolByName(self.context, "portal_catalog")
+        catalog = getToolByName(self.context, 'portal_catalog')
         query = {'meta_type': [i['old_meta_type'] for i in ATCT_LIST.values()]}
-        if HAS_LINGUA_PLONE and 'Language' in catalog.indexes():
-            query['Language'] = 'all'
         brains = catalog(query)
         self._objects_to_be_migrated = len(brains)
         return self._objects_to_be_migrated
@@ -505,7 +481,7 @@ class ATCTMigratorHelpers(BrowserView):
     def estimated_migration_time(self):
         """ Return the estimated migration time """
         total_time = self.objects_to_be_migrated() * ONE_OBJECT_MIGRATION_TIME
-        hours, remainder = divmod(total_time / 1000, 3600)
+        hours, remainder = divmod(total_time // 1000, 3600)
         minutes, seconds = divmod(remainder, 60)
         return {
             'hours': hours,
@@ -524,19 +500,17 @@ class ATCTMigratorHelpers(BrowserView):
         default the migration of subtopics would fail Collections are changed
         to be folderish.
         """
-        catalog = getToolByName(self.context, "portal_catalog")
+        catalog = getToolByName(self.context, 'portal_catalog')
         query = {'meta_type': 'ATTopic'}
         results = []
-        if HAS_LINGUA_PLONE and 'Language' in catalog.indexes():
-            query['Language'] = 'all'
         brains = catalog(query)
         for brain in brains:
             for item in catalog(path={'query': brain.getPath(), 'depth': 1}):
                 results.append(item.getURL())
         if results:
             results = set(results)
-            paths = "\n".join(results)
-            logger.info("Found {0} subtopics at: \n{1}".format(
+            paths = '\n'.join(results)
+            logger.info('Found {0} subtopics at: \n{1}'.format(
                 len(results), paths))
             return results
 
@@ -573,11 +547,11 @@ class ATCTMigratorHelpers(BrowserView):
             </object>
 
         """
-        fti = queryUtility(IDexterityFTI, name="Collection")
-        if fti and fti.content_meta_type == "Dexterity Container":
+        fti = queryUtility(IDexterityFTI, name='Collection')
+        if fti and fti.content_meta_type == 'Dexterity Container':
             return True
         # test for lowercase ttw-type
-        fti = queryUtility(IDexterityFTI, name="collection")
+        fti = queryUtility(IDexterityFTI, name='collection')
         behavior = 'plone.app.contenttypes.behaviors.collection.ICollection'
         if fti and behavior in fti.behaviors:
             logger.warn("You are trying to migrate topic to collection. "
@@ -605,20 +579,6 @@ class ATCTMigratorHelpers(BrowserView):
         return results
 
 
-class ATCTMigratorResults(BrowserView):
-
-    index = ViewPageTemplateFile('atct_migrator_results.pt')
-
-    def results(self):
-        sdm = self.context.session_data_manager
-        session = sdm.getSessionData(create=True)
-        results = session.get("atct_migrator_results", None)
-        if not results:
-            return False
-        # results['atct_list'] = ATCT_LIST
-        return results
-
-
 class PACInstaller(form.Form):
     """Install p.a.c and redirect to migration-form."""
 
@@ -628,31 +588,26 @@ class PACInstaller(form.Form):
 
     @property
     def pac_installable(self):
-        qi = getToolByName(self.context, "portal_quickinstaller")
-        pac_installed = qi.isProductInstalled('plone.app.contenttypes')
-        pac_installable = qi.isProductInstallable('plone.app.contenttypes')
+        qi = get_installer(self.context)
+        pac_installed = qi.is_product_installed('plone.app.contenttypes')
+        pac_installable = qi.is_product_installable('plone.app.contenttypes')
         return pac_installable and not pac_installed
 
     @property
     def pac_installed(self):
-        qi = getToolByName(self.context, "portal_quickinstaller")
-        return qi.isProductInstalled('plone.app.contenttypes')
+        qi = get_installer(self.context)
+        return qi.is_product_installed('plone.app.contenttypes')
 
     @button.buttonAndHandler(_(u'Install'), name='install')
     def handle_install(self, action):
         """ install p.a.c
         """
         url = self.context.absolute_url()
-        qi = getToolByName(self.context, "portal_quickinstaller")
-        fail = qi.installProduct(
-            'plone.app.contenttypes',
-            profile='plone.app.contenttypes:default',
-            blacklistedSteps=['typeinfo'],
+        portal_setup = getToolByName(self.context, 'portal_setup')
+        portal_setup.runAllImportStepsFromProfile(
+            'profile-plone.app.contenttypes:default',
+            blacklisted_steps=['typeinfo'],
         )
-        if fail:
-            messages = IStatusMessage(self.request)
-            messages.addStatusMessage(fail, type='error')
-            self.request.response.redirect(url)
 
         # For types without any instances we want to instantly
         # replace the AT-FTI's with DX-FTI's.
@@ -662,7 +617,7 @@ class PACInstaller(form.Form):
         self.request.response.redirect(url)
 
     def installTypesWithoutItems(self):
-        catalog = getToolByName(self.context, "portal_catalog")
+        catalog = getToolByName(self.context, 'portal_catalog')
         for types_name in DEFAULT_TYPES:
             if not catalog.unrestrictedSearchResults(portal_type=types_name):
                 installTypeIfNeeded(types_name)
