@@ -1,12 +1,5 @@
 # -*- coding: utf-8 -*-
 from Acquisition import aq_base
-from Products.Archetypes.config import REFERENCE_CATALOG
-from Products.Archetypes.interfaces.referenceable import IReferenceable
-from Products.CMFCore.utils import getToolByName
-from Products.CMFPlone.interfaces import IPloneSiteRoot
-from Products.CMFPlone.utils import safe_hasattr
-from Products.GenericSetup.context import DirectoryImportContext
-from Products.GenericSetup.utils import importObjects
 from archetypes.schemaextender.interfaces import IBrowserLayerAwareExtender
 from archetypes.schemaextender.interfaces import IOrderableSchemaExtender
 from archetypes.schemaextender.interfaces import ISchemaExtender
@@ -15,8 +8,7 @@ from copy import deepcopy
 from plone.app.contentrules.api import assign_rule
 from plone.app.contenttypes.behaviors.leadimage import ILeadImage
 from plone.app.contenttypes.migration.field_migrators import migrate_imagefield
-from plone.app.contenttypes.migration.field_migrators import \
-    migrate_simplefield
+from plone.app.contenttypes.migration.field_migrators import migrate_simplefield  # noqa
 from plone.app.contenttypes.utils import DEFAULT_TYPES
 from plone.app.discussion.conversation import ANNOTATION_KEY as DISCUSSION_KEY
 from plone.app.discussion.interfaces import IConversation
@@ -25,6 +17,7 @@ from plone.app.linkintegrity.handlers import modifiedDexterity
 from plone.app.linkintegrity.handlers import referencedRelationship
 from plone.app.uuid.utils import uuidToObject
 from plone.contentrules.engine.interfaces import IRuleAssignmentManager
+from plone.contentrules.engine.interfaces import IRuleStorage
 from plone.dexterity.interfaces import IDexterityContent
 from plone.dexterity.interfaces import IDexterityFTI
 from plone.portlets.constants import CONTEXT_BLACKLIST_STATUS_KEY
@@ -32,6 +25,14 @@ from plone.portlets.interfaces import ILocalPortletAssignable
 from plone.portlets.interfaces import IPortletAssignmentMapping
 from plone.portlets.interfaces import IPortletManager
 from plone.uuid.interfaces import IUUID
+from Products.Archetypes.config import REFERENCE_CATALOG
+from Products.Archetypes.interfaces.referenceable import IReferenceable
+from Products.CMFCore.utils import getToolByName
+from Products.CMFPlone.interfaces import IPloneSiteRoot
+from Products.CMFPlone.utils import safe_hasattr
+from Products.Five.browser import BrowserView
+from Products.GenericSetup.context import DirectoryImportContext
+from Products.GenericSetup.utils import importObjects
 from z3c.relationfield import RelationValue
 from zc.relation.interfaces import ICatalog
 from zExceptions import NotFound
@@ -44,23 +45,14 @@ from zope.component import queryUtility
 from zope.component.hooks import getSite
 from zope.intid.interfaces import IIntIds
 from zope.lifecycleevent import modified
-from Products.Five.browser import BrowserView
-import json
 
+import json
 import logging
 import os
-import pkg_resources
+
 
 logger = logging.getLogger(__name__)
 
-# Is there a multilingual addon?
-try:
-    pkg_resources.get_distribution('Products.LinguaPlone')
-except pkg_resources.DistributionNotFound:
-    HAS_LINGUA_PLONE = False
-else:
-    HAS_LINGUA_PLONE = True
-
 
 def isSchemaExtended(iface):
     """Return a list of fields added by archetypes.schemaextender
@@ -122,13 +114,19 @@ def installTypeIfNeeded(type_name):
     information from the fti in the profile.
     """
     if type_name not in DEFAULT_TYPES:
-        raise KeyError("%s is not one of the default types" % type_name)
+        raise KeyError('{0} is not one of the default types'.format(type_name))
     portal = getSite()
     tt = getToolByName(portal, 'portal_types')
     fti = tt.getTypeInfo(type_name)
     if IDexterityFTI.providedBy(fti):
-        # the dx-type is already installed
-        return
+        # The dx-type is already installed, so keep it.  But this
+        # might be an old dexterity type of Collection, in which case
+        # it is better to replace it.
+        if type_name != 'Collection':
+            return
+        if fti.klass == 'plone.app.contenttypes.content.Collection':
+            # If the klass is fine, we are happy.
+            return
     if fti:
         tt.manage_delObjects(type_name)
     tt.manage_addTypeInformation('Dexterity FTI', id=type_name)
@@ -178,10 +176,22 @@ def copy_contentrules(source_object, target_object):
         try:
             IRuleAssignmentManager(target_object)
         except TypeError:
-            logger.info("Cound not assign contentrules to {0}".format(
-                target_object.absolute_url()))
+            logger.info(
+                'Cound not assign contentrules to {0}'.format(
+                    target_object.absolute_url()
+                )
+            )
             return
+        rules_storage = getUtility(IRuleStorage)
+        available_rules = [r for r in rules_storage]
         for rule_id in source_assignable:
+            if rule_id not in available_rules:
+                logger.info(
+                    'Contentrule {0} does not exist, skip assignment!'.format(
+                        rule_id
+                    )
+                )
+                continue
             assign_rule(target_object, rule_id)
 
 
@@ -200,8 +210,8 @@ def migrate_leadimage(source_object, target_object):
 
     if ILeadImage(target_object, None) is None:
         # skip if new content does not have the LeadImage-behavior enabled
-        logger.info("Target does not have the behavior 'Lead Image' enabled. "
-                    "Could not migrate collective.leadimage fields.")
+        logger.info('Target does not have the behavior "Lead Image" enabled. '
+                    'Could not migrate collective.leadimage fields.')
         return
 
     acc = source_object.getField(
@@ -223,9 +233,9 @@ def migrate_leadimage(source_object, target_object):
         target_object,
         OLD_CAPTION_FIELD_NAME,
         NEW_CAPTION_FIELD_NAME)
-    logger.info(
-        "Migrating contentlead image for {0}.".format(
-            target_object.absolute_url()))
+    logger.info('Migrating contentlead image for {0}.'.format(
+        target_object.absolute_url())
+    )
 
 
 def migrate_portlets(src_obj, dst_obj):
@@ -271,8 +281,9 @@ def store_references(context):
     all_references = get_all_references(context)
     key = 'ALL_REFERENCES'
     IAnnotations(context)[key] = all_references
-    logger.info('Stored {} relations for later restore.'.format(
-        len(all_references)))
+    logger.info('Stored {0} relations for later restore.'.format(
+        len(all_references))
+    )
 
 
 class ExportAllReferences(BrowserView):
@@ -281,17 +292,29 @@ class ExportAllReferences(BrowserView):
 
     def __call__(self):
         data = get_all_references(self.context)
-        self.request.response.setHeader("Content-type", "application/json")
+        self.request.response.setHeader('Content-type', 'application/json')
         return json.dumps(data)
 
 
+def catalog_get_all(catalog, unique_idx='UID'):
+    """Get all brains from the catalog.
+    """
+    res = [
+        catalog({
+            unique_idx: catalog._catalog.getIndexDataForRID(it)[unique_idx]
+        })[0]
+        for it in catalog._catalog.data
+    ]
+    return res
+
+
 def get_all_references(context):
     results = []
     # Archetypes
     # Get all data from the reference_catalog if it exists
     reference_catalog = getToolByName(context, REFERENCE_CATALOG, None)
     if reference_catalog is not None:
-        for brain in reference_catalog():
+        for brain in catalog_get_all(reference_catalog):
             results.append({
                 'from_uuid': brain.sourceUID,
                 'to_uuid': brain.targetUID,
@@ -303,14 +326,16 @@ def get_all_references(context):
     portal_catalog = getToolByName(context, 'portal_catalog')
     relation_catalog = queryUtility(ICatalog)
     for rel in relation_catalog.findRelations():
-        from_brain = portal_catalog(path=dict(query=rel.from_path, depth=0))
-        to_brain = portal_catalog(path=dict(query=rel.to_path, depth=0))
-        if len(from_brain) > 0 and len(to_brain) > 0:
-            results.append({
-                'from_uuid': from_brain[0].UID,
-                'to_uuid': to_brain[0].UID,
-                'relationship': rel.from_attribute,
-            })
+        if rel.from_path and rel.to_path:
+            from_brain = portal_catalog(path=dict(query=rel.from_path,
+                                                  depth=0))
+            to_brain = portal_catalog(path=dict(query=rel.to_path, depth=0))
+            if len(from_brain) > 0 and len(to_brain) > 0:
+                results.append({
+                    'from_uuid': from_brain[0].UID,
+                    'to_uuid': to_brain[0].UID,
+                    'relationship': rel.from_attribute,
+                })
     return results
 
 
@@ -324,15 +349,19 @@ def restore_references(context):
     for ref in IAnnotations(context)[key]:
         source_obj = uuidToObject(ref['from_uuid'])
         target_obj = uuidToObject(ref['to_uuid'])
+        relationship = ref['relationship']
         if source_obj and target_obj:
             relationship = ref['relationship']
             link_items(context, source_obj, target_obj, relationship)
         else:
             logger.warn(
                 'Could not restore reference from uid '
-                '"%s" to uid "%s" on the context: %s' % (
-                    ref['from_uuid'], ref['to_uuid'],
-                    '/'.join(context.getPhysicalPath())))
+                '"{0}" to uid "{1}" on the context: {2}'.format(
+                    ref['from_uuid'],
+                    ref['to_uuid'],
+                    '/'.join(context.getPhysicalPath())
+                )
+            )
     del IAnnotations(context)[key]
 
 
@@ -392,14 +421,14 @@ def link_items(  # noqa
         # with a link to the target and should not be created manually.
         if source_type == 'AT':
             modifiedArchetype(source_obj, None)
-        if source_type == 'DX' and is_referenceable(source_obj):
+        if source_type == 'DX':
             modifiedDexterity(source_obj, None)
         return
 
     if source_type == 'AT':
         # If there is any Archetypes-content there is also the
-        # reference_catalog and the uid_catalog.
-        # For a site without AT content these might not be there at all.
+        # reference_catalog. For a site without AT content this
+        # might not be there at all.
         reference_catalog = getToolByName(context, REFERENCE_CATALOG)
         uid_catalog = getToolByName(context, 'uid_catalog')
         if target_type == 'DX' and not is_referenceable(target_obj):
@@ -424,9 +453,8 @@ def link_items(  # noqa
 
         field = source_obj.getField(fieldname)
         if field is None:
-            # We can't migrate if it doesn't actually have the field
+            # we can't migrate if it doesn't actually have the field
             return
-
         accessor = field.getAccessor(source_obj)
         existing_at_relations = accessor()
 
@@ -442,7 +470,7 @@ def link_items(  # noqa
         targetUIDs = [ref.targetUID for ref in reference_catalog.getReferences(
             source_obj, relationship)]
         if target_uid in targetUIDs:
-            # Replace relations since is probably broken.
+            # Drop relation since the old ones are most likely broken.
             reference_catalog.deleteReference(
                 source_obj, target_uid, relationship)
 
@@ -458,6 +486,10 @@ def link_items(  # noqa
                 source_obj.absolute_url(), target_obj.absolute_url()))
             return
         # handle dx-relation
+        if relationship == 'translationOf':
+            # LinguaPlone relations make no sense for Dexterity
+            return
+
         intids = getUtility(IIntIds)
         to_id = intids.getId(target_obj)
         existing_dx_relations = getattr(source_obj, fieldname, [])
