@@ -1,26 +1,29 @@
 # -*- coding: utf-8 -*-
-from Products.CMFCore.utils import getToolByName
-from Products.CMFPlone.utils import safe_unicode
-from datetime import datetime
-from plone.app.contenttypes.migration.field_migrators import migrate_filefield
-from plone.app.contenttypes.migration.field_migrators import migrate_imagefield
-from plone.app.contenttypes.migration.field_migrators import \
-    migrate_simplefield
-from plone.app.contenttypes.migration.utils import installTypeIfNeeded
+
+from plone.app.contenttypes.testing import TEST_MIGRATION
 from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_MIGRATION_FUNCTIONAL_TESTING  # noqa
 from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_MIGRATION_TESTING  # noqa
-from plone.app.testing import SITE_OWNER_NAME
-from plone.app.testing import SITE_OWNER_PASSWORD
-from plone.app.testing import TEST_USER_ID
-from plone.app.testing import applyProfile
-from plone.app.testing import login
-from plone.app.testing import setRoles
-from plone.testing.z2 import Browser
+import unittest
 
-import pytz
-import os.path
-import transaction
-import unittest2 as unittest
+if TEST_MIGRATION:
+    from datetime import datetime
+    from plone.app.contenttypes.migration.field_migrators import migrate_filefield
+    from plone.app.contenttypes.migration.field_migrators import migrate_imagefield
+    from plone.app.contenttypes.migration.field_migrators import migrate_simplefield  # noqa
+    from plone.app.contenttypes.migration.utils import installTypeIfNeeded
+    from plone.app.testing import applyProfile
+    from plone.app.testing import login
+    from plone.app.testing import setRoles
+    from plone.app.testing import SITE_OWNER_NAME
+    from plone.app.testing import SITE_OWNER_PASSWORD
+    from plone.app.testing import TEST_USER_ID
+    from plone.testing.z2 import Browser
+    from Products.CMFCore.utils import getToolByName
+    from Products.CMFPlone.utils import safe_unicode
+
+    import os.path
+    import pytz
+    import transaction
 
 
 class MigrateFieldsTest(unittest.TestCase):
@@ -28,6 +31,9 @@ class MigrateFieldsTest(unittest.TestCase):
     layer = PLONE_APP_CONTENTTYPES_MIGRATION_TESTING
 
     def setUp(self):
+        if not TEST_MIGRATION:
+            raise unittest.SkipTest('Migration tests require ATContentTypes')
+
         self.portal = self.layer['portal']
         self.request = self.layer['request']
         setRoles(self.portal, TEST_USER_ID, ['Manager'])
@@ -52,14 +58,18 @@ class MigrateFieldsTest(unittest.TestCase):
 
     def test_migrate_stringfield(self):
         # create content
-        at_document_id = self.portal.invokeFactory('Document',
-                                                   'foo',
-                                                   title="Foo document")
+        at_document_id = self.portal.invokeFactory(
+            'Document',
+            'foo',
+            title='Foo document',
+        )
         # register p.a.contenttypes profile
         applyProfile(self.portal, 'plone.app.contenttypes:default')
-        dx_document_id = self.portal.invokeFactory('Document',
-                                                   'bar',
-                                                   title="Bar document")
+        dx_document_id = self.portal.invokeFactory(
+            'Document',
+            'bar',
+            title='Bar document',
+        )
         at_document = self.portal[at_document_id]
         dx_document = self.portal[dx_document_id]
         migrate_simplefield(at_document, dx_document, 'title', 'title')
@@ -67,15 +77,19 @@ class MigrateFieldsTest(unittest.TestCase):
 
     def test_migrate_richtextfield(self):
         # create content
-        at_document_id = self.portal.invokeFactory('Document',
-                                                   'foo',
-                                                   title="Foo document",
-                                                   text="Some foo html text")
+        at_document_id = self.portal.invokeFactory(
+            'Document',
+            'foo',
+            title='Foo document',
+            text='Some foo html text',
+        )
         # register p.a.contenttypes profile
         applyProfile(self.portal, 'plone.app.contenttypes:default')
-        dx_document_id = self.portal.invokeFactory('Document',
-                                                   'bar',
-                                                   title="Bar document")
+        dx_document_id = self.portal.invokeFactory(
+            'Document',
+            'bar',
+            title='Bar document',
+        )
         at_document = self.portal[at_document_id]
         dx_document = self.portal[dx_document_id]
         self.assertEqual(dx_document.text, None)
@@ -84,15 +98,19 @@ class MigrateFieldsTest(unittest.TestCase):
 
     def test_migrate_listfield(self):
         # create content
-        at_document_id = self.portal.invokeFactory('Document',
-                                                   'foo',
-                                                   title="Foo document",
-                                                   subject=['aaa', 'bbb'])
+        at_document_id = self.portal.invokeFactory(
+            'Document',
+            'foo',
+            title='Foo document',
+            subject=['aaa', 'bbb'],
+        )
         # register p.a.contenttypes profile
         applyProfile(self.portal, 'plone.app.contenttypes:default')
-        dx_document_id = self.portal.invokeFactory('Document',
-                                                   'bar',
-                                                   title="Bar document")
+        dx_document_id = self.portal.invokeFactory(
+            'Document',
+            'bar',
+            title='Bar document',
+        )
         at_document = self.portal[at_document_id]
         dx_document = self.portal[dx_document_id]
         migrate_simplefield(at_document, dx_document, 'subject', 'subject',)
@@ -103,14 +121,16 @@ class MigrateFieldsTest(unittest.TestCase):
         at_newsitem_id = self.portal.invokeFactory(
             'News Item',
             'foo',
-            title="Foo news",
-            image=test_image_data)
+            title='Foo news',
+            image=test_image_data,
+        )
         # register p.a.contenttypes profile
         applyProfile(self.portal, 'plone.app.contenttypes:default')
         dx_newsitem_id = self.portal.invokeFactory(
             'News Item',
             'bar',
-            title="Bar news")
+            title='Bar news',
+        )
         at_newsitem = self.portal[at_newsitem_id]
         dx_newsitem = self.portal[dx_newsitem_id]
         self.assertEqual(dx_newsitem.image, None)
@@ -120,15 +140,19 @@ class MigrateFieldsTest(unittest.TestCase):
 
     def test_migrate_filefield(self):
         test_file_data = self.get_test_file_data()
-        at_file_id = self.portal.invokeFactory('File',
-                                               'foo',
-                                               title="Foo file",
-                                               file=test_file_data)
+        at_file_id = self.portal.invokeFactory(
+            'File',
+            'foo',
+            title='Foo file',
+            file=test_file_data,
+        )
         # register p.a.contenttypes profile
         applyProfile(self.portal, 'plone.app.contenttypes:default')
-        dx_file_id = self.portal.invokeFactory('File',
-                                               'bar',
-                                               title="Bar file")
+        dx_file_id = self.portal.invokeFactory(
+            'File',
+            'bar',
+            title='Bar file',
+        )
         at_file = self.portal[at_file_id]
         dx_file = self.portal[dx_file_id]
         self.assertEqual(dx_file.file, None)
@@ -141,6 +165,9 @@ class MigrateCustomATTest(unittest.TestCase):
     layer = PLONE_APP_CONTENTTYPES_MIGRATION_TESTING
 
     def setUp(self):
+        if not TEST_MIGRATION:
+            raise unittest.SkipTest('Migration tests require ATContentTypes')
+
         self.portal = self.layer['portal']
         self.request = self.layer['request']
         setRoles(self.portal, TEST_USER_ID, ['Manager'])
@@ -153,11 +180,11 @@ class MigrateCustomATTest(unittest.TestCase):
 
     def createCustomATDocument(self, id, parent=None):
         from Products.Archetypes.atapi import StringField, TextField
-        from Products.ATContentTypes.interface import IATDocument
+        from Products.ATContentTypes.interfaces import IATDocument
         from archetypes.schemaextender.interfaces import ISchemaExtender
         from archetypes.schemaextender.field import ExtensionField
         from zope.component import getGlobalSiteManager
-        from zope.interface import implements
+        from zope.interface import implementer
 
         # create schema extension
         class ExtensionTextField(ExtensionField, TextField):
@@ -166,8 +193,8 @@ class MigrateCustomATTest(unittest.TestCase):
         class ExtensionStringField(ExtensionField, StringField):
             """ derivative of text for extending schemas """
 
+        @implementer(ISchemaExtender)
         class SchemaExtender(object):
-            implements(ISchemaExtender)
             fields = [
                 ExtensionTextField('textExtended',
                                    ),
@@ -187,10 +214,13 @@ class MigrateCustomATTest(unittest.TestCase):
 
         # create content
         container = parent or self.portal
-        container.invokeFactory('Document', id,
-                                title="Foo document",
-                                stringExtended="foo text",
-                                textExtended='foo extended rich text')
+        container.invokeFactory(
+            'Document',
+            id,
+            title='Foo document',
+            stringExtended='foo text',
+            textExtended='foo extended rich text',
+        )
         at_document = container[id]
 
         # unregister adapter assure test isolation
@@ -202,13 +232,13 @@ class MigrateCustomATTest(unittest.TestCase):
         from plone.app.contenttypes.migration.migration import migrateCustomAT
         from plone.app.contenttypes.interfaces import INewsItem
         at_document = self.createCustomATDocument('foo-document')
-        qi = self.portal.portal_quickinstaller
         # install pac but only install News Items
-        qi.installProduct(
-            'plone.app.contenttypes',
-            profile='plone.app.contenttypes:default',
-            blacklistedSteps=['typeinfo'])
-        installTypeIfNeeded("News Item")
+        portal_setup = getToolByName(self.portal, 'portal_setup')
+        portal_setup.runAllImportStepsFromProfile(
+            'profile-plone.app.contenttypes:default',
+            blacklisted_steps=['typeinfo'],
+        )
+        installTypeIfNeeded('News Item')
         fields_mapping = (
             {'AT_field_name': 'textExtended',
              'AT_field_type': 'Products.Archetypes.Field.TextField',
@@ -225,8 +255,8 @@ class MigrateCustomATTest(unittest.TestCase):
         dx_newsitem = self.portal['foo-document']
         self.assertTrue(INewsItem.providedBy(dx_newsitem))
         self.assertTrue(dx_newsitem is not at_document)
-        self.assertEquals(at_document.textExtended(), dx_newsitem.text.raw)
-        self.assertEquals(at_document.stringExtended, dx_newsitem.title)
+        self.assertEqual(at_document.textExtended(), dx_newsitem.text.raw)
+        self.assertEqual(at_document.stringExtended, dx_newsitem.title)
 
     def test_migrate_atevent_to_dxnewsitem(self):
         """Tests the custom migration by migrating a default type. It is not
@@ -265,13 +295,13 @@ class MigrateCustomATTest(unittest.TestCase):
         oldTZ = os.environ.get('TZ', None)
         os.environ['TZ'] = 'Asia/Tbilisi'
 
-        qi = self.portal.portal_quickinstaller
         # install pac but only install News Items
-        qi.installProduct(
-            'plone.app.contenttypes',
-            profile='plone.app.contenttypes:default',
-            blacklistedSteps=['typeinfo'])
-        installTypeIfNeeded("News Item")
+        portal_setup = getToolByName(self.portal, 'portal_setup')
+        portal_setup.runAllImportStepsFromProfile(
+            'profile-plone.app.contenttypes:default',
+            blacklisted_steps=['typeinfo'],
+        )
+        installTypeIfNeeded('News Item')
         fields_mapping = (
             {'AT_field_name': 'text',
              'AT_field_type': 'Products.Archetypes.Field.TextField',
@@ -292,10 +322,10 @@ class MigrateCustomATTest(unittest.TestCase):
         dx_newsitem = self.portal['event']
         self.assertTrue(INewsItem.providedBy(dx_newsitem))
         self.assertTrue(dx_newsitem is not at_event)
-        self.assertEquals(
+        self.assertEqual(
             safe_unicode(at_event.getText()),
             dx_newsitem.text.output)
-        self.assertEquals(
+        self.assertEqual(
             at_event.contactName,
             dx_newsitem.image_caption)
 
@@ -339,13 +369,13 @@ class MigrateCustomATTest(unittest.TestCase):
         os.environ['TZ'] = TZ
         timezone = pytz.timezone(TZ)
 
-        qi = self.portal.portal_quickinstaller
         # install pac but only install Event
-        qi.installProduct(
-            'plone.app.contenttypes',
-            profile='plone.app.contenttypes:default',
-            blacklistedSteps=['typeinfo'])
-        installTypeIfNeeded("Event")
+        portal_setup = getToolByName(self.portal, 'portal_setup')
+        portal_setup.runAllImportStepsFromProfile(
+            'profile-plone.app.contenttypes:default',
+            blacklisted_steps=['typeinfo'],
+        )
+        installTypeIfNeeded('Event')
         fields_mapping = (
             {'AT_field_name': 'startDate',
              'AT_field_type': 'Products.Archetypes.Field.DateTimeField',
@@ -385,17 +415,17 @@ class MigrateCustomATTest(unittest.TestCase):
         dx_event = self.portal['event']
         self.assertTrue(IEvent.providedBy(dx_event))
         self.assertTrue(dx_event is not at_event)
-        self.assertEquals(safe_unicode(
+        self.assertEqual(safe_unicode(
             at_event.getText()), dx_event.text.output)
-        self.assertEquals(at_event.eventUrl, dx_event.event_url)
-        self.assertEquals(at_event.contactEmail, dx_event.contact_email)
-        self.assertEquals(at_event.contactName, dx_event.contact_name)
-        self.assertEquals(at_event.contactPhone, dx_event.contact_phone)
-        self.assertEquals(at_event.attendees, dx_event.attendees)
-        self.assertEquals(
+        self.assertEqual(at_event.eventUrl, dx_event.event_url)
+        self.assertEqual(at_event.contactEmail, dx_event.contact_email)
+        self.assertEqual(at_event.contactName, dx_event.contact_name)
+        self.assertEqual(at_event.contactPhone, dx_event.contact_phone)
+        self.assertEqual(at_event.attendees, dx_event.attendees)
+        self.assertEqual(
             dx_event.start,
             timezone.localize(datetime.strptime(start, FORMAT)))
-        self.assertEquals(
+        self.assertEqual(
             dx_event.end, timezone.localize(datetime.strptime(end, FORMAT)))
         if oldTZ:
             os.environ['TZ'] = oldTZ
@@ -409,15 +439,15 @@ class MigrateCustomATTest(unittest.TestCase):
         at_document = self.createCustomATDocument('foo-document')
         at_text = (
             u'Some | field is | pipe-delimited | in the field\n'
-            )
+        )
         at_document.setText(at_text)
-        qi = self.portal.portal_quickinstaller
         # install pac but only install News Items
-        qi.installProduct(
-            'plone.app.contenttypes',
-            profile='plone.app.contenttypes:default',
-            blacklistedSteps=['typeinfo'])
-        installTypeIfNeeded("News Item")
+        portal_setup = getToolByName(self.portal, 'portal_setup')
+        portal_setup.runAllImportStepsFromProfile(
+            'profile-plone.app.contenttypes:default',
+            blacklisted_steps=['typeinfo'],
+        )
+        installTypeIfNeeded('News Item')
         fields_mapping = (
             {'AT_field_name': 'text',
              'DX_field_name': 'creators',
@@ -429,8 +459,8 @@ class MigrateCustomATTest(unittest.TestCase):
         dx_newsitem = self.portal['foo-document']
         self.assertTrue(INewsItem.providedBy(dx_newsitem))
         self.assertTrue(dx_newsitem is not at_document)
-        self.assertEquals(4, len(dx_newsitem.creators))
-        self.assertEquals(at_document.Title(), dx_newsitem.title)
+        self.assertEqual(4, len(dx_newsitem.creators))
+        self.assertEqual(at_document.Title(), dx_newsitem.title)
 
 
 def some_field_migrator(src_obj, dst_obj, src_fieldname, dst_fieldname):
@@ -451,26 +481,29 @@ class CustomMigrationFunctionalTests(unittest.TestCase):
     layer = PLONE_APP_CONTENTTYPES_MIGRATION_FUNCTIONAL_TESTING
 
     def setUp(self):
+        if not TEST_MIGRATION:
+            raise unittest.SkipTest('Migration tests require ATContentTypes')
+
         app = self.layer['app']
         self.portal = self.layer['portal']
         self.request = self.layer['request']
         self.request['ACTUAL_URL'] = self.portal.absolute_url()
         self.request['URL'] = self.portal.absolute_url()
-        self.catalog = getToolByName(self.portal, "portal_catalog")
+        self.catalog = getToolByName(self.portal, 'portal_catalog')
         self.portal.acl_users.userFolderAddUser('admin',
                                                 'secret',
                                                 ['Manager'],
                                                 [])
         login(self.portal, 'admin')
         self.portal.portal_workflow.setDefaultChain(
-            "simple_publication_workflow")
+            'simple_publication_workflow')
         self.portal_url = self.portal.absolute_url()
 
         self.browser = Browser(app)
         self.browser.handleErrors = False
         self.browser.addHeader(
             'Authorization',
-            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD,)
+            'Basic {0}:{1}'.format(SITE_OWNER_NAME, SITE_OWNER_PASSWORD,)
         )
 
     def tearDown(self):
@@ -492,10 +525,10 @@ class CustomMigrationFunctionalTests(unittest.TestCase):
         self.portal.doc1.setTitle(u'Ein Döcument')
         self.portal.doc1.setDescription(u'Sö was')
         transaction.commit()
-        self.browser.open('%s/@@pac_installer' % self.portal_url)
+        self.browser.open('{0}/@@pac_installer'.format(self.portal_url))
         self.browser.getControl('Install').click()
         # open custom-migration-form
-        self.browser.open('%s/@@custom_migration' % self.portal_url)
+        self.browser.open('{0}/@@custom_migration'.format(self.portal_url))
         results = self.browser.contents
         self.assertIn('Custom types migration control panel', results)
         self.assertIn('<input type="hidden" name="Document:list" value="text__type__Products.Archetypes.Field.TextField" />', results)  # noqa
@@ -508,8 +541,14 @@ class CustomMigrationFunctionalTests(unittest.TestCase):
         # run migration
         self.browser.getControl(name='form.button.Migrate').click()
         results = self.browser.contents
-        self.assertIn('Migration applied succesfully for 1 "Document" items.', results)  # noqa
-        self.assertIn('Migration applied succesfully for 1 "Event" items.', results)  # noqa
+        self.assertIn(
+            'Migration applied successfully for 1 "Document" items.',
+            results,
+        )
+        self.assertIn(
+            'Migration applied successfully for 1 "Event" items.',
+            results,
+        )
         self.assertIn('No content to migrate.', results)
         link1 = self.portal['doc1']
         self.assertEqual(link1.portal_type, 'Link')
@@ -535,10 +574,10 @@ class CustomMigrationFunctionalTests(unittest.TestCase):
         self.portal.news1.setTitle(u'Ein News Item')
         self.portal.news1.setDescription(u'Sö was')
         transaction.commit()
-        self.browser.open('%s/@@pac_installer' % self.portal_url)
+        self.browser.open('{0}/@@pac_installer'.format(self.portal_url))
         self.browser.getControl('Install').click()
         # open custom-migration-form
-        self.browser.open('%s/@@custom_migration' % self.portal_url)
+        self.browser.open('{0}/@@custom_migration'.format(self.portal_url))
         results = self.browser.contents
         self.assertIn('Custom types migration control panel', results)
         self.assertIn('<input type="hidden" name="News_space_Item:list" value="text__type__Products.Archetypes.Field.TextField" />', results)  # noqa
@@ -551,8 +590,14 @@ class CustomMigrationFunctionalTests(unittest.TestCase):
         # run migration
         self.browser.getControl(name='form.button.Migrate').click()
         results = self.browser.contents
-        self.assertIn('Migration applied succesfully for 1 "News Item" items.', results)  # noqa
-        self.assertIn('Migration applied succesfully for 1 "Event" items.', results)  # noqa
+        self.assertIn(
+            'Migration applied successfully for 1 "News Item" items.',
+            results,
+        )
+        self.assertIn(
+            'Migration applied successfully for 1 "Event" items.',
+            results,
+        )
         self.assertIn('No content to migrate.', results)
         link1 = self.portal['news1']
         self.assertEqual(link1.portal_type, 'Link')
