@@ -1,25 +1,26 @@
 # -*- coding: utf-8 -*-
-from DateTime import DateTime
-from Products.CMFCore.utils import getToolByName
-from plone.app.contenttypes.behaviors.collection import ICollection
-from plone.app.contenttypes.migration.topics import migrate_topics
-from plone.app.contenttypes.testing import \
-    PLONE_APP_CONTENTTYPES_MIGRATION_TESTING
-from plone.app.querystring.queryparser import parseFormquery
-from plone.app.testing import applyProfile
-from plone.app.testing import login
-from plone.dexterity.content import Container
-from plone.dexterity.interfaces import IDexterityFTI
-from zope.component import queryUtility
-from zope.interface import implementer
-
+from plone.app.contenttypes.testing import TEST_MIGRATION
+from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_MIGRATION_TESTING  # noqa
 import unittest
 
+if TEST_MIGRATION:
+    from DateTime import DateTime
+    from plone.app.contenttypes.behaviors.collection import ICollection
+    from plone.app.contenttypes.migration.topics import migrate_topics
+    from plone.app.querystring.queryparser import parseFormquery
+    from plone.app.testing import applyProfile
+    from plone.app.testing import login
+    from plone.dexterity.content import Container
+    from plone.dexterity.interfaces import IDexterityFTI
+    from Products.CMFCore.utils import getToolByName
+    from zope.component import queryUtility
+    from zope.interface import implementer
 
-@implementer(ICollection)
-class FolderishCollection(Container):
-    """Test subclass for folderish ``Collections``.
-    """
+
+    @implementer(ICollection)
+    class FolderishCollection(Container):
+        """Test subclass for folderish ``Collections``.
+        """
 
 
 class MigrateTopicsIntegrationTest(unittest.TestCase):
@@ -27,26 +28,29 @@ class MigrateTopicsIntegrationTest(unittest.TestCase):
     layer = PLONE_APP_CONTENTTYPES_MIGRATION_TESTING
 
     def setUp(self):
+        if not TEST_MIGRATION:
+            raise unittest.SkipTest('Migration tests require ATContentTypes')
+
         self.portal = self.layer['portal']
         self.request = self.layer['request']
         self.request['ACTUAL_URL'] = self.portal.absolute_url()
         self.request['URL'] = self.portal.absolute_url()
-        self.catalog = getToolByName(self.portal, "portal_catalog")
+        self.catalog = getToolByName(self.portal, 'portal_catalog')
         self.portal.acl_users.userFolderAddUser('admin',
                                                 'secret',
                                                 ['Manager'],
                                                 [])
         login(self.portal, 'admin')
         self.portal.portal_workflow.setDefaultChain(
-            "simple_publication_workflow")
-        self.portal.invokeFactory("Topic", "topic", title="Topic")
-        self.portal.invokeFactory("Folder", "folder", title="Folder")
+            'simple_publication_workflow')
+        self.portal.invokeFactory('Topic', 'topic', title='Topic')
+        self.portal.invokeFactory('Folder', 'folder', title='Folder')
 
     def run_migration(self):
         migrate_topics(self.portal)
 
     def add_criterion(self, index, criterion, value=None):
-        name = '%s_%s' % (index, criterion)
+        name = '{0}_{1}'.format(index, criterion)
         self.portal.topic.addCriterion(index, criterion)
         crit = self.portal.topic.getCriterion(name)
         if value is not None:
@@ -80,6 +84,7 @@ class MigrateTopicsIntegrationTest(unittest.TestCase):
         self.assertEqual(self.portal.topic.portal_type, 'Collection')
         self.assertEqual(new.limit, 42)
         self.assertEqual(new.customViewFields, ('Title', 'Type'))
+        self.assertEqual(self.portal.topic.text.raw, u'<p>Hello</p>')
 
     def test_migrate_layout(self):
         self.portal.topic.setLayout('folder_summary_view')
@@ -95,7 +100,7 @@ class MigrateTopicsIntegrationTest(unittest.TestCase):
 
     def test_migrate_nested_topic(self):
         self.portal.portal_types['Topic'].filter_content_types = False
-        self.portal.topic.invokeFactory("Topic", "subtopic", title="Sub Topic")
+        self.portal.topic.invokeFactory('Topic', 'subtopic', title='Sub Topic')
         applyProfile(self.portal, 'plone.app.contenttypes:default')
         fti = queryUtility(IDexterityFTI, name='Collection')
         # switch our a custom folderish base-class for collections
@@ -136,7 +141,7 @@ class MigrateTopicsIntegrationTest(unittest.TestCase):
         self.assertEqual(
             self.portal.topic.query,
             [{'i': 'review_state',
-              'o': 'plone.app.querystring.operation.selection.is',
+              'o': 'plone.app.querystring.operation.selection.any',
               'v': 'published'}]
         )
 
@@ -327,7 +332,6 @@ class MigrateTopicsIntegrationTest(unittest.TestCase):
         # Check that the resulting query does not give an error.
         self.portal.topic.results
 
-    @unittest.skip('P4 has no plone.app.querystring.operation.selection.any')
     def test_ATListCriterion(self):
         # The new-style queries do not currently offer the possibility
         # to choose if the given values should be joined with 'or' or
@@ -351,7 +355,7 @@ class MigrateTopicsIntegrationTest(unittest.TestCase):
                           'v': ('foo', 'bar')})
         self.assertEqual(query[1],
                          {'i': 'portal_type',
-                          'o': 'plone.app.querystring.operation.selection.is',
+                          'o': 'plone.app.querystring.operation.selection.any',
                           'v': ('Document', 'Folder')})
 
         # Check that the resulting query does not give an error.
@@ -401,7 +405,7 @@ class MigrateTopicsIntegrationTest(unittest.TestCase):
         # Collections support multiple paths since
         # plone.app.querystring 1.2.0.
         login(self.portal, 'admin')
-        self.portal.invokeFactory("Folder", "folder2", title="Folder 2")
+        self.portal.invokeFactory('Folder', 'folder2', title='Folder 2')
         crit = self.add_criterion(
             'path',
             'ATPathCriterion',
@@ -428,7 +432,7 @@ class MigrateTopicsIntegrationTest(unittest.TestCase):
         # Collections support multiple paths since
         # plone.app.querystring 1.2.0.
         login(self.portal, 'admin')
-        self.portal.invokeFactory("Folder", "folder2", title="Folder 2")
+        self.portal.invokeFactory('Folder', 'folder2', title='Folder 2')
         crit = self.add_criterion(
             'path',
             'ATPathCriterion',
@@ -531,7 +535,7 @@ class MigrateTopicsIntegrationTest(unittest.TestCase):
         self.assertEqual(
             query,
             [{'i': 'portal_type',
-              'o': 'plone.app.querystring.operation.selection.is',
+              'o': 'plone.app.querystring.operation.selection.any',
               'v': ('Document', 'Folder')}]
         )
 
@@ -548,15 +552,15 @@ class MigrateTopicsIntegrationTest(unittest.TestCase):
         applyProfile(self.portal, 'plone.app.contenttypes:default')
         self.run_migration()
         query = self.portal.topic.query
-        self.assertEqual(query,
-                         [{'i': 'portal_type',
-                           'o': 'plone.app.querystring.operation.selection.is',
-                           'v': ('Collection',)}])
+        self.assertEqual(
+            query,
+            [{'i': 'portal_type',
+              'o': 'plone.app.querystring.operation.selection.any',
+              'v': ('Collection',)}])
 
         # Check that the resulting query does not give an error.
         self.portal.topic.results
 
-    @unittest.skip('P4 has no plone.app.querystring.operation.selection.any')
     def test_ATSelectionCriterion(self):
         # The new-style queries do not currently offer the possibility
         # to choose if the given values should be joined with 'or' or
@@ -586,7 +590,7 @@ class MigrateTopicsIntegrationTest(unittest.TestCase):
                           'v': ('foo', 'bar')})
         self.assertEqual(query[1],
                          {'i': 'portal_type',
-                          'o': 'plone.app.querystring.operation.selection.is',
+                          'o': 'plone.app.querystring.operation.selection.any',
                           'v': ('Document', 'Collection')})
 
         # Check that the resulting query does not give an error.
@@ -604,10 +608,11 @@ class MigrateTopicsIntegrationTest(unittest.TestCase):
         applyProfile(self.portal, 'plone.app.contenttypes:default')
         self.run_migration()
         query = self.portal.topic.query
-        self.assertEqual(query,
-                         [{'i': 'portal_type',
-                           'o': 'plone.app.querystring.operation.selection.is',
-                           'v': ['Document', 'Folder']}])
+        self.assertEqual(
+            query,
+            [{'i': 'portal_type',
+              'o': 'plone.app.querystring.operation.selection.any',
+              'v': ['Document', 'Folder']}])
 
         # Check that the resulting query does not give an error.
         self.portal.topic.results
